"""empty message

Revision ID: 8a1f2192cab7
Revises:
Create Date: 2025-03-04 17:07:14.030943

"""

from typing import Sequence, Union

import sqlalchemy as sa
import sqlmodel
from alembic import op

# revision identifiers, used by Alembic.
revision: str = "8a1f2192cab7"
down_revision: Union[str, None] = None
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "friend",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sqlmodel.sql.sqltypes.AutoString(), nullable=False),
        sa.Column("age", sa.Integer(), nullable=False),
        sa.Column("years_known", sa.Integer(), nullable=False),
        sa.Column("owes_me", sa.Boolean(), nullable=False),
        sa.Column("has_a_dog", sa.Boolean(), nullable=False),
        sa.Column("spouse_is_annoying", sa.Boolean(), nullable=False),
        sa.Column(
            "met",
            sa.DateTime(timezone=True),
            server_default=sa.text("(CURRENT_TIMESTAMP)"),
            nullable=True,
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("friend")
    # ### end Alembic commands ###
